﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using TCPMessageServer;

namespace TCPMessageServerDemo
{
    public partial class DemoClient : Form
    {
        
        private MessageClient _client;

        public DemoClient()
        {
            InitializeComponent();
            _client = new MessageClient(9897);
            _client.ClientIDChanged += _client_ClientIDChanged;
            _client.Connected += _client_Connected;
            _client.Disconnected += _client_Disconnected;
            _client.MessageReceived += _client_MessageReceived;
            _client.OnError += _client_OnError;
            _client.ClientLogin += _client_ClientLogin;
            _client.ClientLoginFailed += _client_ClientLoginFailed;
            _client.ClientLoginSuccess += _client_ClientLoginSuccess;
            _client.FileReceive += _client_FileReceive;
            _client.FileReceived += _client_FileReceived;
            _client.StartListening();
        }

        void _client_FileReceived(object sender, TransferFileEventArgs e)
        {
            if (this.InvokeRequired)
            {
                FileReceivedHandler eh = new FileReceivedHandler(_client_FileReceived);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                if (e.Total == e.Received)
                {
                    label1.Visible = false;
                    progressBar1.Visible = false;
                    lblSpeed.Visible = false;
                }
                else
                {
                    if (progressBar1.Maximum != (int)e.Total)
                        progressBar1.Maximum = (int)e.Total;

                    if (!progressBar1.Visible)
                        progressBar1.Visible = true;

                    progressBar1.Value = (int)e.Received;

                    lblSpeed.Text = e.TransferSpeed + "\n" + e.TimeRemaining.ToString() + " remaining";
                }
            }
        }

        void _client_FileReceive(object sender, TransferFileEventArgs e)
        {
            if (this.InvokeRequired)
            {
                FileReceivedHandler eh = new FileReceivedHandler(_client_FileReceive);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                e.FileName = "C:\\Projects\\TCPMessageServer\\Files\\ClientFiles\\" + e.FileName;

                label1.Text = "Receiving File";
                label1.Visible = true;
                progressBar1.Visible = true;
                lblSpeed.Visible = true;
            }
        }

        void _client_ClientLoginSuccess(object sender, EventArgs e)
        {
            AddToLog("Succesfully Logged In");
        }

        void _client_ClientLoginFailed(object sender, EventArgs e)
        {
            AddToLog("Failed to Login");
        }

        void _client_ClientLogin(object sender, ClientLoginArgs e)
        {
            AddToLog("Login Required");

            if (InvokeRequired)
            {
                ClientLoginHandler eh = new ClientLoginHandler(_client_ClientLogin);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                LoginForm frmLogin = new LoginForm();

                if (frmLogin.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    e.Username = frmLogin.txtUsername.Text;
                    e.Password = frmLogin.txtPassword.Text;
                }
            }
        }

        void _client_OnError(object sender, ErrorEventArgs e)
        {
            AddToLog(String.Format("Error: {0}", e.Error.Message));
            e.Continue = true;
        }

        void _client_MessageReceived(object sender, TCPMessageServer.Message message)
        {
            if (this.InvokeRequired)
            {
                MessageReceivedEventHandler eh = new MessageReceivedEventHandler(_client_MessageReceived);
                this.Invoke(eh, new object[] { sender, message });
            }
            else
            {
                if (message.Type == MessageType.Command)
                {
                    switch (message.Title)
                    {
                        case "CLEAR LOGS":
                            lstMessage.Items.Clear();
                            break;
                    }
                }

                if (message.Type == MessageType.File)
                    AddToLog(String.Format("{0} - {1}", message.Type.ToString(), message.Title));
                else
                    AddToLog(String.Format("{0} - {1} - {2}", message.Type.ToString(), message.Title, message.Contents));
            }
        }

        void _client_Disconnected(object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                EventHandler eh = new EventHandler(_client_Disconnected);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                btnStop.Enabled = false;
                btnStart.Enabled = true;
                btnBroadcast.Enabled = false;
                btnSendMessage.Enabled = false;
                btnSendFile.Enabled = false;
                AddToLog("Client Disconnected");
            }
        }

        void _client_Connected(object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                EventHandler eh = new EventHandler(_client_Connected);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                btnStart.Enabled = false;
                btnStop.Enabled = true;
                btnBroadcast.Enabled = true;
                btnSendMessage.Enabled = true;
                btnSendFile.Enabled = true;
                AddToLog("Client Connected");
            }
        }

        void _client_ClientIDChanged(object sender, EventArgs e)
        {
            AddToLog(String.Format("New Client ID: {0}", _client.ClientID));
        }

        private delegate void LogHandler(string s);

        private void AddToLog(string s)
        {
            if (this.InvokeRequired)
            {
                LogHandler eh = new LogHandler(AddToLog);
                this.Invoke(eh, new object[] { s });
            }
            else
            {
                lstMessage.SelectedIndex = lstMessage.Items.Add(s);
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            _client.StartListening();
            _client.IgnoreBroadcasts = !cbIgnoreBroadcast.Checked;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            _client.StopListening();
        }

        private void btnSendMessage_Click(object sender, EventArgs e)
        {
            _client.SendMessage(new TCPMessageServer.Message("Hello Server", "Message Content", MessageType.Info));
        }

        private void btnBroadcast_Click(object sender, EventArgs e)
        {
            _client.SendMessage(new TCPMessageServer.Message("Hello Clients", String.Format("Message From {0}", _client.ClientID), MessageType.Broadcast));
        }

        private void cbIgnoreBroadcast_CheckedChanged(object sender, EventArgs e)
        {
            _client.IgnoreBroadcasts = cbIgnoreBroadcast.Checked;
        }

        private void btnSendFile_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                _client.SendFile(openFileDialog1.FileName);
        }

        private void btnReceiveFile_Click(object sender, EventArgs e)
        {
            _client.ReceiveFile("TestFile.docx");
        }

        private void btnBufferSize_Click(object sender, EventArgs e)
        {
            if (btnBufferSize.Text == "Large Buffer")
            {
                btnBufferSize.Text = "Small Buffer";
                _client.BufferSize = FileBufferSize.Size4096;
            }
            else
            {
                btnBufferSize.Text = "Large Buffer";
                _client.BufferSize = FileBufferSize.Size1024;
            }
        }

        private void btnSendUserMessage_Click(object sender, EventArgs e)
        {
            _client.SendMessage(new TCPMessageServer.Message("user", "Hello user", MessageType.User));
        }
    }
}
