﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using TCPMessageServer;

namespace TCPMessageServerDemo
{
    public partial class frmServer : Form
    {
        private MessageServer _server;

        public frmServer()
        {
            InitializeComponent();

            tbBroadcastSpeed.Value = timer1.Interval / 10;

            _server = new MessageServer(9897);
            _server.AcceptFiles = true;
            _server.AllowClientConnect += _server_AllowClientConnect;
            _server.ClientConnected += _server_ClientConnected;
            _server.ClientDisconnected += _server_ClientDisconnected;
            _server.MessageReceived += _server_MessageReceived;
            _server.OnError += _server_OnError;
            _server.Started += _server_Started;
            _server.Stopped += _server_Stopped;
            _server.ClientLogin += _server_ClientLogin;
            _server.FileReceive += _server_FileReceive;
            _server.FileReceived += _server_FileReceived;
            _server.MaximumFileSize = 0;
        }

        void _server_FileReceive(object sender, TransferFileEventArgs e)
        {
            if (this.InvokeRequired)
            {
                FileReceivedHandler eh = new FileReceivedHandler(_server_FileReceive);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                e.FileName = "C:\\Projects\\TCPMessageServer\\Files\\" + e.FileName;
                label4.Text = "Receiving File";
                label4.Visible = true;
                progressBar1.Visible = true;
                lblSpeed.Visible = true;
                AddToLog(String.Format("File request received from client: {1} to transfer file {0}", e.FileName, e.ClientID));
            }
        }

        void _server_FileReceived(object sender, TransferFileEventArgs e)
        {
            if (this.InvokeRequired)
            {
                FileReceivedHandler eh = new FileReceivedHandler(_server_FileReceived);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                if (e.Total == e.Received)
                {
                    label4.Visible = false;
                    progressBar1.Visible = false;
                    lblSpeed.Visible = false;
                }
                else
                {
                    if (progressBar1.Maximum != (int)e.Total)
                        progressBar1.Maximum = (int)e.Total;

                    progressBar1.Value = (int)e.Received;

                    lblSpeed.Text = e.TransferSpeed + "\n" + e.TimeRemaining.ToString() + " remaining";

                }
            }
        }

        void _server_ClientLogin(object sender, ClientLoginArgs e)
        {
            e.LoggedIn = (e.Username == "user" && e.Password == "masterkey");

            if (e.LoggedIn)
                AddToLog(String.Format("Client Logged In, Username: {0}; Password: {1}", e.Username, e.Password));
            else
                AddToLog(String.Format("Client Failed To Log In, Username: {0}; Password: {1}", e.Username, e.Password));
        }

        void _server_Stopped(object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                EventHandler eh = new EventHandler(_server_Stopped);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                btnStart.Enabled = true;
                btnStop.Enabled = false;
                btnClearClientLogs.Enabled = false;
                cbAutoBroadcast.Enabled = false;
                tbBroadcastSpeed.Enabled = false;
                btnSpawnClient.Enabled = false;
                btnSendMessage.Enabled = false;
            }
        }

        void _server_Started(object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                EventHandler eh = new EventHandler(_server_Started);
                this.Invoke(eh, new object[] { sender, e });
            }
            else
            {
                btnStop.Enabled = true;
                btnStart.Enabled = false;
                btnClearClientLogs.Enabled = true;
                cbAutoBroadcast.Enabled = true;
                tbBroadcastSpeed.Enabled = true;
                btnSpawnClient.Enabled = true;
                btnSendMessage.Enabled = true;
            }
        }

        void _server_OnError(object sender, ErrorEventArgs e)
        {
            AddToLog(String.Format("Error: {0}", e.Error.Message));
            e.Continue = true;
        }

        void _server_MessageReceived(object sender, TCPMessageServer.Message message)
        {
            AddToLog(String.Format("{0} - {1} - {2} - {3}", message.Type.ToString(), message.ClientID, message.Title, message.Contents));

            if (message.Type != MessageType.File)
            {
                message.Type = MessageType.Acknowledge;
                _server.SendMessage(message, false);
            }
        }

        void _server_ClientDisconnected(object sender, ClientEventArgs e)
        {
            AddToLog(String.Format("Client Disconnected: {0} - {1}", e.IPAddress.ToString(), e.ClientID));
        }

        void _server_ClientConnected(object sender, ClientEventArgs e)
        {
            AddToLog(String.Format("Client Connected: {0} - {1}", e.IPAddress.ToString(), e.ClientID));
        }

        void _server_AllowClientConnect(object sender, ClientAllowConnectEventArgs e)
        {
            AddToLog(String.Format("Allow Client Connect: {0}", e.IPAddress.ToString()));
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            AddToLog("Attempting to Start");
            _server.Start();
            AddToLog(String.Format("Started: {0}", _server.Running.ToString()));
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            AddToLog("Attempting to Stop");
            timer1.Enabled = false;
            _server.Stop();
        }

        private delegate void LogHandler(string s);

        private void AddToLog(string s)
        {
            if (this.InvokeRequired)
            {
                LogHandler eh = new LogHandler(AddToLog);
                this.Invoke(eh, new object[] { s });
            }
            else
                lstMessage.SelectedIndex = lstMessage.Items.Add(s);
        }

        private void btnSpawnClient_Click(object sender, EventArgs e)
        {
            DemoClient newClient = new DemoClient();
            newClient.Show(this);
        }

        private void frmServer_FormClosing(object sender, FormClosingEventArgs e)
        {
            //must stop server before closing
            e.Cancel = _server.Running;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            _server.SendMessage(new TCPMessageServer.Message("Message", "Message From Server", MessageType.Info));
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            _server.SendMessage(new TCPMessageServer.Message("Auto Broadcast", "Timed broadcast from server", MessageType.Broadcast));
        }

        private void cbAutoBroadcast_CheckedChanged(object sender, EventArgs e)
        {
            timer1.Enabled = cbAutoBroadcast.Checked;
        }

        private void trackBar1_ValueChanged(object sender, EventArgs e)
        {
            timer1.Interval = tbBroadcastSpeed.Value * 10;
        }

        private void btnClearClientLogs_Click(object sender, EventArgs e)
        {
            _server.SendMessage(new TCPMessageServer.Message("CLEAR LOGS", "Server Says!!!", MessageType.Command));
        }

        private void cbLoginRequired_CheckedChanged(object sender, EventArgs e)
        {
            _server.LoginRequird = cbLoginRequired.Checked;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            _server.MaxClientConnections = (ushort)numericUpDown1.Value;
        }

        private void spinTimeout_ValueChanged(object sender, EventArgs e)
        {
            _server.ClientTimeOut = (ushort)spinTimeout.Value;
        }

        private void btnSendFile_Click(object sender, EventArgs e)
        {

        }

    }
}
